/* Main Calendar Widget
 * FIXME: for now this is just a wrapper around gnome-cal.[ch]
 * Copyright (C) 2002  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: calendar-widget.c,v 1.4 2002/11/03 20:15:37 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <glib.h>

#include <gnome.h>

#include <mimedir/mimedir-vcal.h>
#include <mimedir/mimedir-vcomponent.h>
#include <mimedir/mimedir-vtodo.h>

#include "calendar-widget.h"
#include "gncal-day-panel.h"
#include "gnome-cal.h"


static void calendar_widget_init	(CalendarWidget		*widget);
static void calendar_widget_dispose	(GObject		*object);
static void calendar_widget_setup	(CalendarWidget		*widget);
static void calendar_widget_class_init	(CalendarWidgetClass	*klass);


struct _CalendarWidgetPriv {
	GtkWidget	*calendar;

	gchar		*filename;
	time_t		 file_read_time;
};


static GtkVBoxClass *parent_class = NULL;

/*
 * Class and Instance Setup
 */

GType
calendar_widget_get_type (void)
{
	static GType calendar_widget_type = 0;

	if (!calendar_widget_type) {
		static const GTypeInfo calendar_widget_info = {
			sizeof (CalendarWidgetClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) calendar_widget_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (CalendarWidget),
			1,    /* n_preallocs */
			(GInstanceInitFunc) calendar_widget_init,
		};

		calendar_widget_type = g_type_register_static (GTK_TYPE_VBOX,
							       "CalendarWidget",
							       &calendar_widget_info,
							       0);
	}

	return calendar_widget_type;
}

static void
calendar_widget_class_init (CalendarWidgetClass *klass)
{
	GObjectClass *gobject_class;

	g_return_if_fail (klass != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET_CLASS (klass));

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->dispose = calendar_widget_dispose;

	parent_class = g_type_class_peek_parent (klass);
}

static void
calendar_widget_init (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	widget->priv = g_new0 (CalendarWidgetPriv, 1);

	calendar_widget_setup (widget);
}

static void
calendar_widget_dispose (GObject *object)
{
	CalendarWidget *widget;
	CalendarWidgetPriv *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (object));

	widget = CALENDAR_WIDGET (object);
	priv = widget->priv;

	if (priv) {
		g_free (priv->filename);
		g_free (priv);
		widget->priv = NULL;
	}

	G_OBJECT_CLASS (parent_class)->dispose (object);
}

/*
 * Setup
 */

static void
calendar_widget_setup (CalendarWidget *widget)
{
	CalendarWidgetPriv *priv;

	priv = widget->priv;

	priv->calendar = gnome_calendar_new ();
	gtk_container_add (GTK_CONTAINER (widget), priv->calendar);
	gtk_widget_show (priv->calendar);
}

/*
 * Public Methods
 */

GtkWidget *
calendar_widget_new (void)
{
	CalendarWidget *widget;

	widget = g_object_new (CALENDAR_TYPE_WIDGET, NULL);

	return GTK_WIDGET (widget);
}

void
calendar_widget_next (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_next (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_previous (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_previous (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_today (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_goto_today (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_goto (CalendarWidget *widget, struct tm *tm)
{
	time_t t;

	t = mktime (tm);

	/* FIXME: make gnome_calendar_goto accept dates outside the Unix range */
	if (t >= 0)
		gnome_calendar_goto (GNOME_CALENDAR (widget->priv->calendar), t);
}

void
calendar_widget_calendar_new (CalendarWidget *widget)
{
	CalendarWidgetPriv *priv;

	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	priv = widget->priv;

	/* FIXME: implement */

	g_free (priv->filename);
	priv->filename = NULL;
	priv->file_read_time = -1;
}

static void
calendar_widget_load_from_vcal_list (CalendarWidget *widget, GList *list)
{
	GList *item;
	GList *todo_list = NULL;

	for (item = list; item != NULL; item = g_list_next (item)) {
		MIMEDirVCal *vcal;
		GSList *list2, *item2;

		g_assert (MIMEDIR_IS_VCAL (item->data));
		vcal = MIMEDIR_VCAL (item->data);

		list2 = mimedir_vcal_get_component_list (vcal);

		for (item2 = list2; item2 != NULL; item2 = g_slist_next (item2)) {
			g_assert (item2->data != NULL && MIMEDIR_IS_VCOMPONENT (item2->data));

			if (MIMEDIR_IS_VTODO (item2->data))
				todo_list = g_list_append (todo_list, item2->data);
			else
				calendar_add_object (GNOME_CALENDAR (widget->priv->calendar)->cal, MIMEDIR_VCOMPONENT (item2->data));
		}

		mimedir_vcal_free_component_list (list2);
	}

	/* Todo */

	{
		TodoList *todo;

		todo = gncal_day_panel_get_todo_list (GNCAL_DAY_PANEL (GNOME_CALENDAR (widget->priv->calendar)->day_view));
		todo_list_set_list (todo, todo_list);
		g_list_free (todo_list);
	}
}

gboolean
calendar_widget_calendar_load (CalendarWidget *widget,
			       const gchar *filename,
			       GError **error)
{
	GError *err = NULL;
	CalendarWidgetPriv *priv;
	Calendar *cal;
	GList *list;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	priv = widget->priv;

	/* Filename setup */

	if (!filename)
		filename = priv->filename;
	g_assert (filename != NULL);

	cal = GNOME_CALENDAR (priv->calendar)->cal;

	calendar_free (cal);

	priv->file_read_time = time (NULL);

	calendar_set_day ();

	list = mimedir_vcal_read_list (filename, &err);
	if (err) {
		g_propagate_error (error, err);
		return FALSE;
	}

	calendar_widget_load_from_vcal_list (widget, list);

	mimedir_vcal_free_list (list);

	gnome_calendar_update_all (GNOME_CALENDAR (priv->calendar), NULL, 0);

	return TRUE;
}

gboolean
calendar_widget_calendar_load_default (CalendarWidget *widget,
				       GError **error)
{
	gchar *filename = NULL;
	gboolean ret;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	filename = g_build_filename (g_get_home_dir (),
				     ".gnome/user-cal.vcf", /* FIXME: .gnome2/user-cal.ics */
				     NULL);
	ret = calendar_widget_calendar_load (widget, filename, error);
	g_free (filename);

	return ret;
}

static gboolean
calendar_widget_save_real (CalendarWidget *widget, const gchar *filename, GError **error)
{
	GError *err = NULL;
	CalendarWidgetPriv *priv;
	gchar *backup_name;
	GIOChannel *channel;
	struct stat s;

	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	priv = widget->priv;

	/* Create backup file */

	backup_name = g_strconcat (filename, "~", NULL);

	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		if (rename (filename, backup_name) == -1) {
			g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), _("Error while creating backup file %s: %s"), backup_name, strerror (errno));
			g_free (backup_name);
			return FALSE;
		}
	}

	/* Write file */

	channel = g_io_channel_new_file (filename, "w", &err);
	if (channel) {
		MIMEDirVCal *vcal;

		vcal = calendar_widget_get_vcalendar (widget);

		mimedir_vcal_write_to_channel (vcal, channel, &err);

		g_object_unref (G_OBJECT (vcal));
		g_io_channel_unref (channel);
	}

	/* Error handling: try to restore backup file */

	if (err) {
		rename (backup_name, filename);
		g_free (backup_name);

		g_propagate_error (error, err);

		return FALSE;
	}

	g_free (backup_name);

	/* Update calendar */

	if (priv->filename != filename) {
		g_free (priv->filename);
		priv->filename = g_strdup (filename);
	}
	if (stat (filename, &s) == -1)
		priv->file_read_time = time (NULL);
	else
		priv->file_read_time = s.st_mtime;

	return TRUE;
}

gboolean
calendar_widget_calendar_save (CalendarWidget *widget, const gchar *file, GError **error)
{
	CalendarWidgetPriv *priv;
	const gchar *filename = NULL;
        struct stat s;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	priv = widget->priv;

	/* Retrieve the file name */

	if (file)
		filename = file;
	else
		filename = "foo.ics"; /* FIXME: file dialog */

	/* Check whether the file has changed since loading */

	if (stat (filename, &s) == -1) {
		if (errno != ENOENT) {
			g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), _("Could not stat %s: %s"), filename, strerror (errno));
			return FALSE;
		}
	} else if (s.st_mtime > priv->file_read_time){
		GtkWidget *box;

		box = gtk_message_dialog_new (NULL, 0,
					      GTK_MESSAGE_INFO,
					      GTK_BUTTONS_NONE,
					      _("File %s has changed since it was loaded. Save anyways and overwrite any changes?"),
					      filename);
		gtk_dialog_add_buttons (GTK_DIALOG (box),
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
					NULL);
		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);

		switch (gtk_dialog_run (GTK_DIALOG (box))) {
		case GTK_RESPONSE_OK:
			break;
		default:
			return FALSE;
		}

		gtk_widget_destroy (box);
	}

	return calendar_widget_save_real (widget, filename, error);
}


MIMEDirVCal *
calendar_widget_get_vcalendar (CalendarWidget *widget)
{
	CalendarWidgetPriv *priv;
	MIMEDirVCal *vcal;
	GnomeCalendar *gcal;
	GList *list;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);

	priv = widget->priv;

	vcal = mimedir_vcal_new ();

	gcal = GNOME_CALENDAR (widget->priv->calendar);

	/* Events */

	mimedir_vcal_add_component_list (vcal, gcal->cal->events);

	/* Todo */

	{
		TodoList *todo;

		todo = gncal_day_panel_get_todo_list (GNCAL_DAY_PANEL (GNOME_CALENDAR (priv->calendar)->day_view));
		list = todo_list_get_list (todo);
		mimedir_vcal_add_component_list (vcal, list);
		g_list_free (list);
	}

	/* Journal */

	mimedir_vcal_add_component_list (vcal, gcal->cal->journal);

	return vcal;
}

void
calendar_widget_set_time (CalendarWidget *widget, time_t t)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_goto (GNOME_CALENDAR (widget->priv->calendar), t);
}

time_t
calendar_widget_get_time (CalendarWidget *widget)
{
	g_return_val_if_fail (widget != NULL, -1);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), -1);

	return gnome_calendar_current_display (GNOME_CALENDAR (widget->priv->calendar));
}
