/*
 * GnomeCalendar widget
 * Copyright (C) 1998 the Free Software Foundation
 *
 * Author: Miguel de Icaza (miguel@kernel.org)
 *
 * Changed in 2002 by Sebastian Rittau <srittau@jroger.in-berlin.de>
 * for the GNOME 2 port.
 */

#ifndef GNOME_CALENDAR_APP_H
#define GNOME_CALENDAR_APP_H

#include <glib.h>

#include <gnome.h>

#include <mimedir/mimedir-vcomponent.h>

#include "calendar.h"

G_BEGIN_DECLS

#define GNOME_TYPE_CALENDAR		(gnome_calendar_get_type ())
#define GNOME_CALENDAR(obj)		(GTK_CHECK_CAST(obj, GNOME_TYPE_CALENDAR, GnomeCalendar))
#define GNOME_CALENDAR_CLASS(class)	(GTK_CHECK_CAST_CLASS(class, GNOME_TYPE_CALENDAR, GnomeCalendarClass))
#define GNOME_IS_CALENDAR(obj)		(GTK_CHECK_TYPE(obj, GNOME_TYPE_CALENDAR))

/* FIXME: Most of this should go into a private struct */
typedef struct {
	GtkNotebook parent;

	Calendar    *cal;
	time_t      current_day_display;
	time_t      current_week_display;
	time_t      current_month_display;
	time_t      current_year_display;

	GtkWidget   *day_view;
	GtkWidget   *week_view;
	GtkWidget   *month_view;
	GtkWidget   *year_view;
	GtkWidget   *year_view_sw;
	void        *event_editor;
} GnomeCalendar;

typedef struct {
	GtkNotebookClass parent_class;
} GnomeCalendarClass;

GType      gnome_calendar_get_type         	(void);
GtkWidget *gnome_calendar_new              	(void);
void       gnome_calendar_add_object       	(GnomeCalendar *gcal,
						 MIMEDirVComponent *comp);
void       gnome_calendar_remove_object    	(GnomeCalendar *gcal,
						 MIMEDirVComponent *comp);
void       gnome_calendar_next             	(GnomeCalendar *gcal);
void       gnome_calendar_previous         	(GnomeCalendar *gcal);
void       gnome_calendar_goto             	(GnomeCalendar *gcal,
						 time_t new_time);
void       gnome_calendar_dayjump          	(GnomeCalendar *gcal,
						 time_t time);
/* Jumps to the current day */
void       gnome_calendar_goto_today            (GnomeCalendar *gcal);
void       gnome_calendar_tag_calendar          (GnomeCalendar *cal,
						 GtkCalendar *gtk_cal);
char      *gnome_calendar_get_current_view_name (GnomeCalendar *gcal);
void       gnome_calendar_set_view              (GnomeCalendar *gcal,
						 const gchar *page_name);

/* Flags is a bitmask of CalObjectChange values */
void       gnome_calendar_object_changed        (GnomeCalendar *gcal,
						 MIMEDirVComponent *comp,
						 int flags);

/* Return the currently displayed time on the currently displayed page */
time_t     gnome_calendar_current_display	(GnomeCalendar *gcal);

void       gnome_calendar_update_all		(GnomeCalendar *cal, MIMEDirVComponent *component, int flags);

G_END_DECLS

#endif
