/* GNOME Card Editor Edit List Object
 * Copyright (C) 2002  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: cardeditor-editlist.c,v 1.2 2002/08/24 17:02:04 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "cardeditor-editlist.h"
#include "cardeditor-marshal.h"


/* FIXME: list sortable, drag & drop */


enum {
	SIGNAL_SELECTION_CHANGED,
	SIGNAL_ADD_ITEM,
	SIGNAL_REMOVE_ITEM,
	SIGNAL_LAST
};

struct _CardEditorEditListPriv {
	GtkWidget *editor_frame;
	GtkWidget *editor;
	GtkWidget *list_window;
	GtkWidget *list;
	GtkWidget *remove_button;
};


static GtkHBoxClass *parent_class = NULL;

static gint cardeditor_editlist_signals[SIGNAL_LAST] = { 0, 0, 0 };


static void cardeditor_editlist_class_init	(CardEditorEditListClass *klass);
static void cardeditor_editlist_init		(CardEditorEditList *list);
static void cardeditor_editlist_dispose		(GObject *object);

static void cardeditor_editlist_setup		(CardEditorEditList *list);

/*
 * Class and Object Management
 */

GType
cardeditor_editlist_get_type (void)
{
	static GType cardeditor_editlist_type = 0;

	if (!cardeditor_editlist_type) {
		static const GTypeInfo cardeditor_editlist_info = {
			sizeof (CardEditorEditListClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) cardeditor_editlist_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (CardEditorEditList),
			1,    /* n_preallocs */
			(GInstanceInitFunc) cardeditor_editlist_init,
		};

		cardeditor_editlist_type = g_type_register_static (GTK_TYPE_HBOX,
								   "CardEditorEditList",
								   &cardeditor_editlist_info,
								   0);
	}

	return cardeditor_editlist_type;
}


static void
cardeditor_editlist_class_init (CardEditorEditListClass *klass)
{
	GObjectClass *gobject_class;

	g_return_if_fail (klass != NULL);
	g_return_if_fail (CARDEDITOR_IS_EDITLIST_CLASS (klass));

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->dispose = cardeditor_editlist_dispose;

	parent_class = g_type_class_peek_parent (klass);

	/* Signals */

	cardeditor_editlist_signals[SIGNAL_SELECTION_CHANGED] =
		g_signal_new ("selection-changed",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (CardEditorEditListClass, selection_changed),
			      NULL, NULL,
			      cardeditor_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	cardeditor_editlist_signals[SIGNAL_ADD_ITEM] =
		g_signal_new ("add-item",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (CardEditorEditListClass, add_item),
			      NULL, NULL,
			      cardeditor_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	cardeditor_editlist_signals[SIGNAL_REMOVE_ITEM] =
		g_signal_new ("remove-item",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (CardEditorEditListClass, remove_item),
			      NULL, NULL,
			      cardeditor_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
}


static void
cardeditor_editlist_init (CardEditorEditList *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CARDEDITOR_IS_EDITLIST (widget));

	widget->priv = g_new0 (CardEditorEditListPriv, 1);

	cardeditor_editlist_setup (widget);
}


static void
cardeditor_editlist_dispose (GObject *object)
{
	CardEditorEditList *list;

	g_return_if_fail (object != NULL);
	g_return_if_fail (CARDEDITOR_IS_EDITLIST (object));

	list = CARDEDITOR_EDITLIST (object);

	g_free (list->priv);
	list->priv = NULL;

	G_OBJECT_CLASS (parent_class)->dispose (object);
}

/*
 * Callbacks
 */

static void
cardeditor_editlist_selection_changed (CardEditorEditList *list, GtkTreeSelection *selection)
{
	gboolean selected;

	selected = gtk_tree_selection_get_selected (selection, NULL, NULL);

	/* Send signal */

	g_signal_emit (G_OBJECT (list), cardeditor_editlist_signals[SIGNAL_SELECTION_CHANGED], 0);

	/* Change sensibility */

	gtk_widget_set_sensitive (list->priv->remove_button, selected);
	if (list->priv->editor)
		gtk_widget_set_sensitive (list->priv->editor, selected);

}


static void
cardeditor_editlist_add_cb (CardEditorEditList *list, GtkButton *button)
{
	g_signal_emit (G_OBJECT (list), cardeditor_editlist_signals[SIGNAL_ADD_ITEM], 0);
}


static void
cardeditor_editlist_remove_cb (CardEditorEditList *list, GtkButton *button)
{
	g_signal_emit (G_OBJECT (list), cardeditor_editlist_signals[SIGNAL_REMOVE_ITEM], 0);
}

/*
 * Internal Methods
 */

static void
cardeditor_editlist_setup (CardEditorEditList *list)
{
	GtkWidget *w, *box, *box2, *frame;

	gtk_box_set_spacing (GTK_BOX (list), GNOME_PAD);

	frame = gtk_frame_new (NULL);
	gtk_widget_show (frame);
	gtk_box_pack_start (GTK_BOX (list), frame, TRUE, TRUE, 0);
	list->priv->editor_frame = frame;

	box = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD);
	gtk_widget_show (box);
	gtk_box_pack_start (GTK_BOX (list), box, TRUE, TRUE, 0);

	box2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (box2), GTK_SHADOW_IN);
	gtk_widget_show (box2);
	gtk_box_pack_start (GTK_BOX (box), box2, TRUE, TRUE, 0);
	list->priv->list_window = box2;

	box2 = gtk_hbutton_box_new ();
	gtk_button_box_set_layout (GTK_BUTTON_BOX (box2), GTK_BUTTONBOX_EDGE);
	gtk_widget_show (box2);
	gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 0);

	w = gtk_button_new_from_stock (GTK_STOCK_NEW);
	g_signal_connect_swapped (G_OBJECT (w), "clicked", G_CALLBACK (cardeditor_editlist_add_cb), list);
	gtk_widget_show (w);
	gtk_box_pack_start (GTK_BOX (box2), w, FALSE, FALSE, 0);

	w = gtk_button_new_from_stock (GTK_STOCK_DELETE);
	g_signal_connect_swapped (G_OBJECT (w), "clicked", G_CALLBACK (cardeditor_editlist_remove_cb), list);
	gtk_widget_set_sensitive (w, FALSE);
	gtk_widget_show (w);
	gtk_box_pack_start (GTK_BOX (box2), w, FALSE, FALSE, 0);
	list->priv->remove_button = w;
}

/*
 * External Methods
 */

void
cardeditor_editlist_set_editor (CardEditorEditList *list, GtkWidget *editor)
{
	CardEditorEditListPriv *priv;

	g_return_if_fail (list != NULL);
	g_return_if_fail (CARDEDITOR_IS_EDITLIST (list));
	g_return_if_fail (editor != NULL);
	g_return_if_fail (GTK_IS_WIDGET (editor));

	priv = list->priv;

	if (priv->editor)
		gtk_container_remove (GTK_CONTAINER (priv->editor_frame), priv->editor);
	gtk_container_add (GTK_CONTAINER (priv->editor_frame), editor);
	priv->editor = editor;

	if (priv->list) {
		GtkTreeSelection *selection;

		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list));
		cardeditor_editlist_selection_changed (list, selection);
	}
}


void
cardeditor_editlist_set_list (CardEditorEditList *list, GtkTreeView *view)
{
	CardEditorEditListPriv *priv;
	GtkTreeSelection *selection;

	g_return_if_fail (list != NULL);
	g_return_if_fail (CARDEDITOR_IS_EDITLIST (list));
	g_return_if_fail (view != NULL);
	g_return_if_fail (GTK_IS_TREE_VIEW (view));

	priv = list->priv;

	if (priv->list)
		gtk_container_remove (GTK_CONTAINER (priv->list_window), priv->list);
	gtk_container_add (GTK_CONTAINER (priv->list_window), GTK_WIDGET (view));
	priv->list = GTK_WIDGET (view);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
	g_signal_connect_swapped (G_OBJECT (selection), "changed", G_CALLBACK (cardeditor_editlist_selection_changed), list);

	if (priv->editor)
		cardeditor_editlist_selection_changed (list, selection);
}
