/* GnomeCard - a graphical contact manager.
 *
 * dialog.c: This file is part of GnomeCard.
 * $Id: dialog.c,v 1.51 2003/01/12 16:38:58 srittau Exp $
 * 
 * Copyright (C) 1999 The Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changed in 2002 and 2003 by Sebastian Rittau <srittau@jroger.in-berlin.de>
 * for the GNOME 2 port.
 */

/* FIXME: store find dialog settings, and words in session */
/* FIXME: public key handling must be reworked */
/* FIXME: timezone, longitude, latitude, etc. can be turned off */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <gnome.h>

#include <fnmatch.h>

#include "cardeditor-addresses.h"
#include "cardeditor-email.h"
#include "cardeditor-phone.h"
#include "cardlist-widget.h"
#include "dialog.h"
#include "gnomecard.h"
#include "gnomecard-main-window.h"
#include "gnomecard-ui.h"
#include "sort.h"
#include "utils-ui.h"

#include "world.xpm"


typedef struct
{
	GtkWidget *dialog;

	/* Identity */

	GtkWidget *fn;
	GtkWidget *given, *add, *fam, *pre, *suf;

	/* Geographical */

	GtkWidget *tzh, *tzm;

	MIMEDirVCard *vcard;
} GnomeCardEditor;

typedef struct
{
	GtkWidget *entry, *sens;
} GnomeCardFind;

static void gnomecard_prop_close (GtkWidget *widget, gpointer data);
static void gnomecard_take_from_name (GtkWidget *widget, gpointer data);

static void
gnomecard_prop_close (GtkWidget *widget, gpointer data)
{
	/* Cleanup */

	g_object_set_data (G_OBJECT (data), "being_edited", NULL);
}

static void gnomecard_take_from_name(GtkWidget *widget, gpointer data)
{
        GnomeCardEditor *ce;
	char *name;
	
	ce = (GnomeCardEditor *) data;

	name = gnomecard_join_name(gtk_entry_get_text(GTK_ENTRY(ce->pre)),
			 gtk_entry_get_text(GTK_ENTRY(ce->given)),
			 gtk_entry_get_text(GTK_ENTRY(ce->add)),
			 gtk_entry_get_text(GTK_ENTRY(ce->fam)),
			 gtk_entry_get_text(GTK_ENTRY(ce->suf)));
	
	gtk_entry_set_text(GTK_ENTRY(ce->fn), name);
	
	g_free(name);
}

static void
dialog_fn_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *fn;

	fn = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "name", fn, NULL);
}

static void
dialog_given_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "givenname", s, NULL);
}

static void
dialog_middle_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "middlename", s, NULL);
}

static void
dialog_last_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "familyname", s, NULL);
}

static void
dialog_prefix_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "prefix", s, NULL);
}

static void
dialog_suffix_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "suffix", s, NULL);
}

static void
dialog_org_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "organization", s, NULL);
}

static void
dialog_title_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "jobtitle", s, NULL);
}

static void
dialog_bday_changed (GnomeDateEdit *de, MIMEDirVCard *vcard)
{
	time_t t;
	struct tm tm;
	GDate date;

	t = gnome_date_edit_get_time (de);
	localtime_r (&t, &tm);
	g_date_clear (&date, 1);
	g_date_set_dmy (&date, tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900);
	g_object_set (G_OBJECT (vcard), "birthday", &date, NULL);

}

static void
dialog_tz_changed (GtkSpinButton *button, GnomeCardEditor *ce)
{
	MIMEDirVCard *vcard = ce->vcard;
	gint hour, minute;

	hour   = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ce->tzh));
	minute = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ce->tzm));
	if (hour < 0)
		minute = -minute;

	g_object_set (G_OBJECT (vcard), "timezone", hour * 60 + minute, NULL);
}

static void
dialog_lat_changed (GtkSpinButton *button, MIMEDirVCard *vcard)
{
	gdouble d;

	d = gtk_spin_button_get_value (button);

	g_object_set (G_OBJECT (vcard), "latitude", d, NULL);
}

static void
dialog_long_changed (GtkSpinButton *button, MIMEDirVCard *vcard)
{
	gdouble d;

	d = gtk_spin_button_get_value (button);

	g_object_set (G_OBJECT (vcard), "longitude", d, NULL);
}

static void
dialog_cat_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *s;

	s = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "categories", s, NULL);
}

static void
dialog_comment_changed (GtkTextBuffer *buffer, MIMEDirVCard *vcard)
{
	GtkTextIter start, end;
	gchar *s;

	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_get_end_iter   (buffer, &end);

	s = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	g_object_set (G_OBJECT (vcard), "note", s, NULL);
	g_free (s);
}

static void
dialog_url_changed (GtkEntry *entry, MIMEDirVCard *vcard)
{
	const gchar *url;

	url = gtk_entry_get_text (entry);

	g_object_set (G_OBJECT (vcard), "url", url, NULL);
}

static void
dialog_key_changed (GtkTextBuffer *buffer, MIMEDirVCard *vcard)
{
	GtkTextIter start, end;
	gchar *s;

	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_get_end_iter   (buffer, &end);

	s = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	g_object_set (G_OBJECT (vcard), "pubkey", s, NULL);
	g_free (s);
}

static void
dialog_keytype_pgp (GtkToggleButton *button, MIMEDirVCard *vcard)
{
	if (gtk_toggle_button_get_active (button))
		g_object_set (G_OBJECT (vcard), "keytype", MIMEDIR_VCARD_KEY_PGP, NULL);
}

static void
dialog_keytype_x509 (GtkToggleButton *button, MIMEDirVCard *vcard)
{
	if (gtk_toggle_button_get_active (button))
		g_object_set (G_OBJECT (vcard), "keytype", MIMEDIR_VCARD_KEY_X509, NULL);
}

static void
dialog_keytype_other (GtkToggleButton *button, MIMEDirVCard *vcard)
{
	if (gtk_toggle_button_get_active (button))
		g_object_set (G_OBJECT (vcard), "keytype", MIMEDIR_VCARD_KEY_UNKNOWN, NULL);
}

static void
dialog_editor_response (GtkDialog *dialog, gint response, gpointer data)
{
	g_return_if_fail (dialog != NULL);
	g_return_if_fail (GTK_IS_DIALOG (dialog));

	switch (response) {
	case GTK_RESPONSE_REJECT:
		/* FIXME: revert */
		break;
	case GTK_RESPONSE_CLOSE:
		gtk_widget_destroy (GTK_WIDGET (dialog));
		break;
	case GTK_RESPONSE_DELETE_EVENT:
		break;
	case GTK_RESPONSE_HELP:
		ui_show_help (GTK_WINDOW (dialog), "edit");
		break;
	default:
		g_return_if_reached ();
	}
}

void
dialog_edit_card(MIMEDirVCard *vcard)
{
	GdkPixmap *pixmap;
	GtkWidget *prop_box, *notebook;
	GnomeCardEditor *ce;
	GtkWidget *hbox, *hbox2, *vbox, *frame, *table;
	GtkWidget *label, *align, *align2, *pix;
	GtkWidget *radio1, *radio2, *radio3, *button;
	GtkWidget *editor;
	GtkWidget *w;
	GtkObject *adj;
	GtkWidget *nametable;
	GtkTextBuffer *text_buffer;
	GtkStyle *style;
	time_t tmp_time;
	gint fontsize;

	gint timezone, hour, minute;
	gdouble latitude, longitude;
	MIMEDirVCardKeyType keytype;

	gchar *s;
	GDate *date;

	ce = g_object_get_data (G_OBJECT (vcard), "being_edited");
	if (ce) {
		gtk_window_present (GTK_WINDOW (ce->dialog));
		return;
	}

	ce = g_new0(GnomeCardEditor, 1);
	ce->vcard = vcard;
	
	/* Set flag */

	g_object_set_data (G_OBJECT (vcard), "being_edited", ce);

	prop_box = gtk_dialog_new ();
	ce->dialog = prop_box;
	gtk_dialog_add_buttons (GTK_DIALOG (prop_box),
				GTK_STOCK_HELP,  GTK_RESPONSE_HELP,
				GTK_STOCK_UNDO,  GTK_RESPONSE_REJECT,
				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
				NULL);
	g_signal_connect (G_OBJECT (prop_box), "response",
			  G_CALLBACK (dialog_editor_response), NULL);
	g_signal_connect (G_OBJECT (prop_box), "destroy",
			  G_CALLBACK (gnomecard_prop_close), vcard);
	g_object_set_data (G_OBJECT (prop_box), "editor", ce);
	gtk_window_set_wmclass (GTK_WINDOW (prop_box),
				"gnomecard", "gnomecard");

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prop_box)->vbox),
			    notebook, TRUE, TRUE, 0);

	/* Identity notebook page */
	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	label = gtk_label_new(_("Identity"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
	
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	label = gtk_label_new(_("File As:"));
	ce->fn = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "name", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->fn), s);
	g_free (s);
	g_signal_connect (G_OBJECT (ce->fn), "changed", G_CALLBACK (dialog_fn_changed), vcard);
	button = gtk_button_new_with_label(_("Take from Name"));
	g_signal_connect (G_OBJECT (button), "clicked",
			  G_CALLBACK (gnomecard_take_from_name), ce);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), ce->fn, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	/* create name frame */
	frame = gtk_frame_new(_("Name"));	
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, FALSE, 0);

	/* pack all name fields into a table */
	nametable = gtk_table_new (1, 4, FALSE);
	gtk_table_set_col_spacings (GTK_TABLE (nametable), GNOME_PAD_SMALL);
	gtk_table_set_row_spacings (GTK_TABLE (nametable), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (nametable), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), nametable);

	/* first name */
	label = gtk_label_new(_("First:"));
	ce->given = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "givenname", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->given), s);
	g_free (s);
	g_signal_connect (G_OBJECT (ce->given), "changed", G_CALLBACK (dialog_given_changed), vcard);
	align = gtk_alignment_new(0.0, 0.0, 0, 0);
        gtk_container_add (GTK_CONTAINER (align), ce->given);
	gtk_table_attach(GTK_TABLE(nametable), label, 0, 1, 0, 1,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 
			 0, 0);
	gtk_table_attach(GTK_TABLE(nametable), align, 1, 2, 0, 1, 
			 GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 
			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);

	label = gtk_label_new(_("Middle:"));
	ce->add = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "middlename", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->add), s);
	g_free (s);
	g_signal_connect (G_OBJECT (ce->add), "changed", G_CALLBACK (dialog_middle_changed), vcard);
	gtk_table_attach(GTK_TABLE(nametable), label, 2, 3, 0, 1,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 
			 0, 0);
	gtk_table_attach(GTK_TABLE(nametable), ce->add, 3, 4, 0, 1, 
			 GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 
			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);

	label = gtk_label_new(_("Last:"));
	ce->fam = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "familyname", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->fam), s);
	g_free (s);
	g_signal_connect (G_OBJECT (ce->fam), "changed", G_CALLBACK (dialog_last_changed), vcard);
	align = gtk_alignment_new(0.0, 0.0, 0, 0);
        gtk_container_add (GTK_CONTAINER (align), ce->fam);
	gtk_table_attach(GTK_TABLE(nametable), label, 0, 1, 1, 2, 
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 
			 0, 0);
	gtk_table_attach(GTK_TABLE(nametable), align, 1, 4, 1, 2,
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);

	label = gtk_label_new(_("Prefix:"));
	ce->pre = gtk_entry_new ();
	gtk_entry_set_width_chars (GTK_ENTRY (ce->pre), 5);
	g_object_get (G_OBJECT (vcard), "prefix", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->pre), s);
	g_free (s);
	align = gtk_alignment_new(0.0, 0.0, 0, 0);
        gtk_container_add (GTK_CONTAINER (align), ce->pre);
	g_signal_connect (G_OBJECT (ce->pre), "changed", G_CALLBACK (dialog_prefix_changed), vcard);
	gtk_table_attach(GTK_TABLE(nametable), label, 0, 1, 2, 3,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
	gtk_table_attach(GTK_TABLE(nametable), align, 1, 2, 2, 3, 
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 
			 GNOME_PAD_SMALL, 0);

	label = gtk_label_new(_("Suffix:"));
	ce->suf = gtk_entry_new ();
	gtk_entry_set_width_chars (GTK_ENTRY (ce->suf), 5);
	g_object_get (G_OBJECT (vcard), "suffix", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (ce->suf), s);
	g_free (s);
	align = gtk_alignment_new(0.0, 0.0, 0, 0); 
        gtk_container_add (GTK_CONTAINER (align), ce->suf);
	g_signal_connect (G_OBJECT (ce->suf), "changed", G_CALLBACK (dialog_suffix_changed), vcard);
        gtk_table_attach(GTK_TABLE(nametable), label, 2, 3, 2, 3,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
	gtk_table_attach(GTK_TABLE(nametable), align, 3, 4, 2, 3, 
			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
			 GNOME_PAD_SMALL, 0);

	/* organization and internet info share same line */
	/* add organization to identity notetab */
	hbox2 = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);

	frame = gtk_frame_new(_("Organization"));
	gtk_box_pack_start(GTK_BOX(hbox2), frame, TRUE, TRUE, 0);

	table = gtk_table_new (2, 2, FALSE);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), table);

	label = gtk_label_new(_("Name:"));
	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
	w = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "organization", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (w), s);
	g_free (s);
	g_signal_connect (G_OBJECT (w), "changed", G_CALLBACK (dialog_org_changed), vcard);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 
			 0, 0);
	gtk_table_attach(GTK_TABLE(table), w, 1, 2, 0, 1,
			 GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_FILL | GTK_SHRINK,
			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);

	label = gtk_label_new(_("Title:"));
	w = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "jobtitle", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (w), s);
	g_free (s);
	g_signal_connect (G_OBJECT (w), "changed", G_CALLBACK (dialog_title_changed), vcard);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 
			 0, 0);
	gtk_table_attach(GTK_TABLE(table), w, 1, 2, 1, 2,
			 GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_FILL | GTK_SHRINK, 
			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);

	gtk_widget_show_all (vbox);

	/* E-mail page */

	{
		GtkWidget *myw, *mypage, *myhbox;

		mypage = gtk_vbox_new (FALSE, GNOME_PAD);
		gtk_container_set_border_width (GTK_CONTAINER (mypage), GNOME_PAD_SMALL);
		gtk_widget_show (mypage);
	
		/* URL */

		myhbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
		gtk_box_pack_start (GTK_BOX (mypage), myhbox, FALSE, FALSE, 0);
		gtk_widget_show (myhbox);

		label = gtk_label_new_with_mnemonic (_("_Web Page:"));
		gtk_box_pack_start (GTK_BOX (myhbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		myw = gtk_entry_new ();
		g_object_get (G_OBJECT (vcard), "url", &s, NULL);
		if (s)
			gtk_entry_set_text (GTK_ENTRY (myw), s);
		g_free (s);
		gtk_box_pack_start (GTK_BOX (myhbox), myw, TRUE, TRUE, 0);
		gtk_widget_show (myw);
		g_signal_connect (G_OBJECT (myw), "changed", G_CALLBACK (dialog_url_changed), vcard);

		gtk_label_set_mnemonic_widget (GTK_LABEL (label), myw);

		/* E-mail */

		editor = cardeditor_email_new ();
		gtk_box_pack_start (GTK_BOX (mypage), editor, TRUE, TRUE, 0);
		cardeditor_email_edit_vcard (CARDEDITOR_EMAIL (editor), vcard);
		gtk_widget_show (GTK_WIDGET (editor));

		label = gtk_label_new(_("Network"));
		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), mypage, label);
	}

	/* Address page */

	editor = cardeditor_addresses_new ();
	gtk_container_set_border_width (GTK_CONTAINER (editor), GNOME_PAD_SMALL);
	cardeditor_addresses_edit_vcard (CARDEDITOR_ADDRESSES (editor), vcard);
	gtk_widget_show (GTK_WIDGET (editor));

	label = gtk_label_new(_("Addresses"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), editor, label);

	/* Phone number page */

	editor = cardeditor_phone_new ();
	gtk_container_set_border_width (GTK_CONTAINER (editor), GNOME_PAD_SMALL);
	cardeditor_phone_edit_vcard (CARDEDITOR_PHONE (editor), vcard);
	gtk_widget_show (GTK_WIDGET (editor));

	label = gtk_label_new(_("Phone"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), editor, label);

	/* Birth date */
	
	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	label = gtk_label_new(_("Birthdate:"));

	g_object_get (G_OBJECT (vcard), "birthday", &date, NULL);
	if (date && g_date_valid (date)) {
		struct tm tm;
		g_date_to_struct_tm (date, &tm);
		tmp_time = mktime(&tm);
	} else
		tmp_time = time (NULL);

	w = gnome_date_edit_new (tmp_time, FALSE, FALSE);
	g_signal_connect (G_OBJECT (w), "date_changed", G_CALLBACK (dialog_bday_changed), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);

	gtk_widget_show_all (vbox);

	/* Geographical */

	align2 = gtk_alignment_new(0.5, 0.5, 0, 0);
	hbox2 = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
        gtk_container_add (GTK_CONTAINER (align2), hbox2);
	label = gtk_label_new(_("Geographical"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), align2, label);

	pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL,
							gdk_colormap_get_system (),
							NULL, NULL, world_xpm);
	pix = gtk_image_new_from_pixmap (pixmap, NULL);
	g_object_unref (pixmap);
	gtk_box_pack_start(GTK_BOX(hbox2), pix, FALSE, FALSE, 0);

	g_object_get (G_OBJECT (vcard),
		      "timezone",  &timezone,
		      "latitude",  &latitude,
		      "longitude", &longitude,
		      NULL);

	if (timezone >= MIMEDIR_VCARD_TIMEZONE_MIN && timezone <= MIMEDIR_VCARD_TIMEZONE_MAX) {
		hour   = timezone / 60;
		minute = timezone % 60;
	} else {
		hour   = 0;
		minute = 0;
	}
	if (latitude < MIMEDIR_VCARD_LATITUDE_MIN || latitude > MIMEDIR_VCARD_LATITUDE_MAX)
		latitude = 0.0;
	if (longitude < MIMEDIR_VCARD_LATITUDE_MIN || longitude > MIMEDIR_VCARD_LATITUDE_MAX)
		longitude = 0.0;

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(hbox2), vbox, FALSE, FALSE, 0);
	frame = gtk_frame_new(_("Time Zone"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
	align = gtk_alignment_new(0.5, 0.5, 0, 0);
        gtk_container_add (GTK_CONTAINER (align), hbox);
	gtk_container_add(GTK_CONTAINER(frame), align);
	label = gtk_label_new(_("hrs."));
	adj = gtk_adjustment_new((gdouble) hour, -12.0, 12.0, 1.0, 1.0, 3.0);
	ce->tzh = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ce->tzh), TRUE);
	gtk_entry_set_width_chars (GTK_ENTRY (ce->tzh), 3);
	g_signal_connect (G_OBJECT (adj), "value_changed", G_CALLBACK (dialog_tz_changed), ce);
	gtk_box_pack_start(GTK_BOX(hbox), ce->tzh, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	label = gtk_label_new(_("mins."));
	adj = gtk_adjustment_new((gdouble) minute, 0.0, 59.0, 1.0, 1.0, 10.0);
	ce->tzm = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ce->tzm), TRUE);
	gtk_entry_set_width_chars (GTK_ENTRY (ce->tzm), 2);
	g_signal_connect (G_OBJECT (adj), "value_changed", G_CALLBACK (dialog_tz_changed), ce);
	gtk_box_pack_start(GTK_BOX(hbox), ce->tzm, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		
	frame = gtk_frame_new(_("Geographic Position"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
	align = gtk_alignment_new(0.5, 0.5, 0, 0);
        gtk_container_add (GTK_CONTAINER (align), hbox);
	gtk_container_add(GTK_CONTAINER(frame), align);

	label = gtk_label_new(_("lat, "));
	adj = gtk_adjustment_new (latitude, -90, 90, .01, 1, 1);
	w = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (w), TRUE);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (w), 2);
	gtk_entry_set_width_chars (GTK_ENTRY (w), 5);
	g_signal_connect (G_OBJECT (w), "value_changed", G_CALLBACK (dialog_lat_changed), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	label = gtk_label_new(_("lon."));
	adj = gtk_adjustment_new(longitude, -180, 180, .01, 1, 1);
	w = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (w), TRUE);
	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (w), 2);
	gtk_entry_set_width_chars (GTK_ENTRY (w), 6);
	g_signal_connect (G_OBJECT (w), "value_changed", G_CALLBACK (dialog_long_changed), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	gtk_widget_show_all (align2);

	/* Explanatory */
	
	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	label = gtk_label_new(_("Explanatory"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	label = gtk_label_new(_("Categories:"));
	w = gtk_entry_new ();
	g_object_get (G_OBJECT (vcard), "categories", &s, NULL);
	if (s)
		gtk_entry_set_text (GTK_ENTRY (w), s);
	g_free (s);
	g_signal_connect (G_OBJECT (w), "changed", G_CALLBACK (dialog_cat_changed), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	label = gtk_label_new(_("Comment:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
	gtk_widget_show (frame);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
	w = gtk_text_view_new ();
	gtk_text_view_set_editable (GTK_TEXT_VIEW (w), TRUE);
	gtk_container_add (GTK_CONTAINER (frame), w);
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (w));
	g_object_get (G_OBJECT (vcard), "note", &s, NULL);
	if (s)
		gtk_text_buffer_set_text (text_buffer, s, -1);
	g_free (s);
	style = gtk_widget_get_style (w);
	fontsize = pango_font_description_get_size (style->font_desc) / PANGO_SCALE + 1;
	gtk_widget_set_size_request (w, -1, fontsize * 4);
	g_signal_connect (G_OBJECT (text_buffer), "changed", G_CALLBACK (dialog_comment_changed), vcard);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, GNOME_PAD);

	gtk_widget_show_all (vbox);

	/* Security */

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	label = gtk_label_new(_("Security"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

	label = gtk_label_new(_("Public Key:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
	gtk_widget_show (frame);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
	w = gtk_text_view_new ();
	gtk_text_view_set_editable (GTK_TEXT_VIEW (w), TRUE);
	gtk_container_add (GTK_CONTAINER (frame), w);
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (w));
	g_object_get (G_OBJECT (vcard),
		      "pubkey",  &s,
		      "keytype", &keytype,
		      NULL);
	if (s)
		gtk_text_buffer_set_text (text_buffer, s, -1);
	g_free (s);
	gtk_widget_set_size_request (w, -1, fontsize * 6);
		g_signal_connect (G_OBJECT (text_buffer), "changed", G_CALLBACK (dialog_key_changed), vcard);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	radio1 = gtk_radio_button_new_with_label(NULL, _("PGP"));
	g_signal_connect (G_OBJECT (radio1), "toggled", G_CALLBACK (dialog_keytype_pgp), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), radio1, FALSE, FALSE, 0);

	radio2 = gtk_radio_button_new_with_label(
		gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio1)),
		_("X.509"));
	g_signal_connect (G_OBJECT (radio2), "toggled", G_CALLBACK (dialog_keytype_x509), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), radio2, FALSE, FALSE, 0);

	radio3 = gtk_radio_button_new_with_label(
		gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio1)),
		_("Other"));
	g_signal_connect (G_OBJECT (radio3), "toggled", G_CALLBACK (dialog_keytype_other), vcard);
	gtk_box_pack_start(GTK_BOX(hbox), radio3, FALSE, FALSE, 0);

	switch (keytype) {
	case MIMEDIR_VCARD_KEY_PGP:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
		break;
	case MIMEDIR_VCARD_KEY_X509:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);
		break;
	default:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio3), TRUE);
		break;
	}

	gtk_widget_show_all (vbox);

	gtk_widget_show (GTK_WIDGET (prop_box));
}

#if 0 /* FIXME: card */
static void
gnomecard_cancel_dialog(GtkWidget *widget, gint response_id, GtkWidget *dialog)
{
	void *p;

	g_return_if_fail (dialog != NULL);
	g_return_if_fail (GTK_IS_WIDGET (dialog));

	p = g_object_get_data (G_OBJECT (dialog), "editor");
	if (p != NULL)
		g_free(p);
	
	gtk_widget_destroy (dialog);
}

static int gnomecard_match_pattern(char *pattern, char *str, int sens)
{
	char *txt;
	int found = 0;
	
	if (! str)
		return 0;
	
	if (sens)
		txt = str;
	else
		txt = g_utf8_strup(str, -1);
	
	if (fnmatch(pattern, txt, 0) == 0) {
		found = 1;
	}
	
	if (! sens)
		g_free(txt);
	
	return found;
}
#endif

static void
dialog_find_card_real(GnomeCardFind *p, gboolean back)
{
#if 0 /* FIXME: card */
	GList *l = NULL, *k;
	Card *crd;
	char *pattern, *crd_text[16];
	int i, wrapped, found, sens;
	
	found = 0;
	wrapped = 0;

	g_free (gnomecard_find_str);
	gnomecard_find_str = g_strdup(gtk_entry_get_text(GTK_ENTRY(p->entry)));
	pattern = g_strdup_printf ("*%s*", gnomecard_find_str);

	if (GTK_TOGGLE_BUTTON(p->sens)->active)
		gnomecard_find_sens = sens = 1;
	else {
		gchar *s;
		gnomecard_find_sens = sens = 0;
		s = g_utf8_strup (pattern, -1);
		g_free (pattern);
		pattern = s;
	}
	
	l = gnomecard_curr_crd;

	while (l) {
	    if (wrapped != 1)
		l = back ? l->prev : l->next;
	    else
		wrapped ++;
	    
	    if (l) {
		crd = l->data;
		crd_text[0] = crd->fname.str;
		crd_text[1] = crd->name.family;
		crd_text[2] = crd->name.given;
		crd_text[3] = crd->name.additional;
		crd_text[4] = crd->name.prefix;
		crd_text[5] = crd->name.suffix;
		crd_text[6] = crd->title.str;
		crd_text[7] = crd->role.str;
		crd_text[8] = crd->categories.str;
		crd_text[9] = crd->comment.str;
		crd_text[10] = crd->url.str;
		crd_text[11] = crd->org.name;
		crd_text[12] = crd->org.unit1;
		crd_text[13] = crd->org.unit2;
		crd_text[14] = crd->org.unit3;
		crd_text[15] = crd->org.unit4;
		
		for (i = 0; i < sizeof(crd_text)/sizeof(crd_text[0]) && !found; i++)
		    if (gnomecard_match_pattern(pattern, crd_text[i], sens))
			found = 1;
		
		for (k = crd->phone.l; k && !found; k = k->next)
		    if (gnomecard_match_pattern(pattern, 
						((CardPhone *) k->data)->data,
						sens))
			found = 1;
		
		for (k = crd->email.l; k && !found; k = k->next)
		    if (gnomecard_match_pattern(pattern, 
						((CardEMail *) k->data)->data,
						sens))
			found = 1;
		
		for (k = crd->dellabel.l; k && !found; k = k->next)
		    if (gnomecard_match_pattern(pattern, 
						((CardDelLabel *)k->data)->data, sens))
			found = 1;

		for (k = crd->deladdr.l; k && !found; k = k->next)
		    for (i = 0; i < DELADDR_MAX; i++)
			if (gnomecard_match_pattern(pattern, 
						    ((CardDelAddr *) k->data)->data[i], sens))
			    found = 1;
		
		if (found) {
		    gnomecard_scroll_list(l);
		    break;
		}
		
	    }	else if (wrapped) {
		GtkWidget *w;
		
		w = gtk_message_dialog_new (NULL, 0,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_OK,
					    _("No matching record found."));
		GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
		g_signal_connect (G_OBJECT (w), "response",
				  G_CALLBACK (gnomecard_cancel_dialog), w);
		gtk_widget_show(w);
		break;
	    } else {
		GtkWidget *w;
		gchar *msg;
		
		if (back)
			msg = _("Reached first record.\nContinue from the last one?");
		else
			msg = _("Reached last record.\nContinue from the first one?");

		w = gtk_message_dialog_new(NULL,
					   GTK_DIALOG_MODAL,
					   GTK_MESSAGE_QUESTION,
					   GTK_BUTTONS_NONE,
					   msg);
		gtk_dialog_add_buttons (GTK_DIALOG (w),
					_("Abort"),    GTK_RESPONSE_NO,
					_("Continue"), GTK_RESPONSE_YES,
					NULL);

		switch (gtk_dialog_run (GTK_DIALOG (w))) {
		case GTK_RESPONSE_NONE:
		case GTK_RESPONSE_NO:
			l = NULL;
			break;
		case GTK_RESPONSE_YES:
			l = back ? g_list_last(gnomecard_crds) : gnomecard_crds;
			wrapped = 1;
			break;
		default:
			g_return_if_reached ();
		}
		gtk_widget_destroy (w);
	    }
	}
	
	g_free(pattern);
	
	gnome_config_set_bool("/GnomeCard/find/sens",  gnomecard_find_sens);
	gnome_config_set_string("/GnomeCard/find/str",  gnomecard_find_str);
	gnome_config_sync();
#endif
}

static void
dialog_find_response (GtkWidget *widget, gint response_id, GnomeCardFind *cardfind)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (GTK_IS_WIDGET (widget));
	g_return_if_fail (cardfind != NULL);

	switch (response_id) {
	case GTK_RESPONSE_NONE:
	case GTK_RESPONSE_CLOSE:
		break;
	case 1:
		dialog_find_card_real (cardfind, TRUE);
		break;
	case 2:
		dialog_find_card_real (cardfind, FALSE);
		break;
	default:
		g_return_if_reached ();
	}

	g_free (cardfind);
	gtk_widget_destroy (widget);
}

void dialog_find_card(void)
{
	GtkWidget *dialog, *w, *hbox;
	GnomeCardFind *p;

	p = g_malloc(sizeof(GnomeCardFind));
	dialog = gtk_dialog_new_with_buttons (_("Find Card"), NULL, 0,
					      GTK_STOCK_GO_BACK, 1,
					      GTK_STOCK_GO_FORWARD, 2,
					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
					      NULL);
	g_signal_connect (G_OBJECT (dialog), "response",
			  G_CALLBACK (dialog_find_response), p);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog), 2);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
			    FALSE, FALSE, 0);
	w = gtk_label_new (_("Find:"));
	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
	p->entry = gtk_entry_new ();
#if 0 /* FIXME: card */
	gtk_entry_set_text (GTK_ENTRY (p->entry), gnomecard_find_str);
#endif
	gtk_box_pack_start (GTK_BOX (hbox), p->entry, TRUE, TRUE, 0);

	p->sens = gtk_check_button_new_with_label(_("Case Sensitive"));
#if 0 /* FIXME: card */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (p->sens),
				      gnomecard_find_sens);
#endif
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
			    p->sens, FALSE, FALSE, 0);

	gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
	gtk_widget_show (dialog);
}

